from PCRaster import *
from PCRaster.Framework import *

class MyFirstModel(DynamicModel):
  def __init__(self):
    DynamicModel.__init__(self)
    setclone('dem.map')

  def initial(self):
    dem = self.readmap('dem')
    elevationMeteoStation = 208.1
    elevationAboveMeteoStation = dem - elevationMeteoStation
    temperatureLapseRate = 0.005
    self.temperatureCorrection = elevationAboveMeteoStation * temperatureLapseRate
    self.report(self.temperatureCorrection,'tempCor')

    self.snow=0.0

    self.ldd=lddcreate(dem,1e31,1e31,1e31,1e31)
    self.report(self.ldd,'ldd')

    # example how to calculate total precipitation
    self.totPrecip=scalar(0)

  def dynamic(self):
    precipitation = timeinputscalar('precip.tss',1)
    self.report(precipitation,'pFromTss')
    temperatureObserved = timeinputscalar('temp.tss',1)
    self.report(temperatureObserved,'tempObs')
    temperature= temperatureObserved - self.temperatureCorrection
    self.report(temperature,'temp')

    freezing=temperature < 0.0
    self.report(freezing,'freez')
    snowFall=ifthenelse(freezing,precipitation,0.0)
    self.report(snowFall,'snowFall')
    rainFall=ifthenelse(pcrnot(freezing),precipitation,0.0)
    self.report(rainFall,'rain')

    self.snow = self.snow+snowFall
    #self.report(self.snow,'snow')

    potentialMelt = ifthenelse(pcrnot(freezing),temperature*0.01,0)
    self.report(potentialMelt,'pmelt')
    actualMelt = min(self.snow, potentialMelt)
    self.report(actualMelt,'amelt')

    self.snow = self.snow - actualMelt
    self.report(self.snow,'snow')

    runoffGenerated = actualMelt + rainFall
    self.report(runoffGenerated,'rg')

    discharge=accuflux(self.ldd,runoffGenerated*cellarea())
    self.report(discharge,'q')

    # example how to calculate total precipitation
    self.totPrecip=self.totPrecip+precipitation

    if self.currentTimeStep() == 181:
      # this part is run only at the last time step
      # and you can add here the calculations for the static
      # MMF model
      # example
      print 'running MMF model'
      fractionOfTotalPrecipitation=self.totPrecip*0.2
      self.report(fractionOfTotalPrecipitation,'fr')

nrOfTimeSteps=181
myModel = MyFirstModel()
dynamicModel = DynamicFramework(myModel,nrOfTimeSteps)
dynamicModel.run()
